@@ -47,7 +47,6 @@ use uv_normalize::PackageName;
 pub use crate::any::*;
 pub use crate::buildable::*;
 pub use crate::cached::*;
-pub use crate::direct_url::*;
 pub use crate::editable::*;
 pub use crate::error::*;
 pub use crate::file::*;
@@ -55,6 +54,7 @@ pub use crate::hash::*;
 pub use crate::id::*;
 pub use crate::index_url::*;
 pub use crate::installed::*;
+pub use crate::parsed_url::*;
 pub use crate::prioritized_distribution::*;
 pub use crate::resolution::*;
 pub use crate::resolved::*;
@@ -63,7 +63,6 @@ pub use crate::traits::*;
 mod any;
 mod buildable;
 mod cached;
-mod direct_url;
 mod editable;
 mod error;
 mod file;
@@ -71,6 +70,7 @@ mod hash;
 mod id;
 mod index_url;
 mod installed;
+mod parsed_url;
 mod prioritized_distribution;
 mod resolution;
 mod resolved;
