@@ -1,25 +1,26 @@
 use std::collections::btree_map::{BTreeMap, Entry};
 use std::sync::OnceLock;
 
-use chrono::{DateTime, Utc};
+use rkyv::{de::deserializers::SharedDeserializeMap, Deserialize};
 use rustc_hash::FxHashSet;
-use tracing::{instrument, warn};
+use tracing::instrument;
 
 use distribution_filename::{DistFilename, WheelFilename};
 use distribution_types::{
-    Dist, IncompatibleSource, IncompatibleWheel, IndexUrl, PrioritizedDist,
+    Dist, Hash, IncompatibleSource, IncompatibleWheel, IndexUrl, PrioritizedDist,
     SourceDistCompatibility, WheelCompatibility,
 };
 use pep440_rs::{Version, VersionSpecifiers};
-use platform_tags::Tags;
+use platform_tags::{TagCompatibility, Tags};
 use pypi_types::{HashDigest, Yanked};
-use rkyv::{de::deserializers::SharedDeserializeMap, Deserialize};
-use uv_client::{FlatDistributions, OwnedArchive, SimpleMetadata, VersionFiles};
+use uv_client::{OwnedArchive, SimpleMetadata, VersionFiles};
 use uv_configuration::{NoBinary, NoBuild};
 use uv_normalize::PackageName;
+use uv_types::HashStrategy;
 use uv_warnings::warn_user_once;
 
-use crate::{python_requirement::PythonRequirement, yanks::AllowedYanks};
+use crate::flat_index::FlatDistributions;
+use crate::{python_requirement::PythonRequirement, yanks::AllowedYanks, ExcludeNewer};
 
 /// A map from versions to distributions.
 #[derive(Debug)]
@@ -46,7 +47,8 @@ impl VersionMap {
         tags: &Tags,
         python_requirement: &PythonRequirement,
         allowed_yanks: &AllowedYanks,
-        exclude_newer: Option<&DateTime<Utc>>,
+        hasher: &HashStrategy,
+        exclude_newer: Option<&ExcludeNewer>,
         flat_index: Option<FlatDistributions>,
         no_binary: &NoBinary,
         no_build: &NoBuild,
@@ -109,6 +111,7 @@ impl VersionMap {
             .allowed_versions(package_name)
             .cloned()
             .unwrap_or_default();
+        let required_hashes = hasher.get_package(package_name).digests().to_vec();
         Self {
             inner: VersionMapInner::Lazy(VersionMapLazy {
                 map,
@@ -120,6 +123,7 @@ impl VersionMap {
                 python_requirement: python_requirement.clone(),
                 exclude_newer: exclude_newer.copied(),
                 allowed_yanks,
+                required_hashes,
             }),
         }
     }
@@ -299,9 +303,11 @@ struct VersionMapLazy {
     /// exists) is satisfied or not.
     python_requirement: PythonRequirement,
     /// Whether files newer than this timestamp should be excluded or not.
-    exclude_newer: Option<DateTime<Utc>>,
+    exclude_newer: Option<ExcludeNewer>,
     /// Which yanked versions are allowed
     allowed_yanks: FxHashSet<Version>,
+    /// The hashes of allowed distributions.
+    required_hashes: Vec<HashDigest>,
 }
 
 impl VersionMapLazy {
@@ -385,6 +391,7 @@ impl VersionMapLazy {
                             &filename,
                             &version,
                             requires_python,
+                            &hashes,
                             yanked,
                             excluded,
                             upload_time,
@@ -400,6 +407,7 @@ impl VersionMapLazy {
                         let compatibility = self.source_dist_compatibility(
                             &version,
                             requires_python,
+                            &hashes,
                             yanked,
                             excluded,
                             upload_time,
@@ -422,10 +430,12 @@ impl VersionMapLazy {
         simple.dist.get_or_init(get_or_init).as_ref()
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn source_dist_compatibility(
         &self,
         version: &Version,
         requires_python: Option<VersionSpecifiers>,
+        hashes: &[HashDigest],
         yanked: Option<Yanked>,
         excluded: bool,
         upload_time: Option<i64>,
@@ -462,14 +472,32 @@ impl VersionMapLazy {
             }
         }
 
-        SourceDistCompatibility::Compatible
+        // Check if hashes line up. If hashes aren't required, they're considered matching.
+        let hash = if self.required_hashes.is_empty() {
+            Hash::Matched
+        } else {
+            if hashes.is_empty() {
+                Hash::Missing
+            } else if hashes
+                .iter()
+                .any(|hash| self.required_hashes.contains(hash))
+            {
+                Hash::Matched
+            } else {
+                Hash::Mismatched
+            }
+        };
+
+        SourceDistCompatibility::Compatible(hash)
     }
 
+    #[allow(clippy::too_many_arguments)]
     fn wheel_compatibility(
         &self,
         filename: &WheelFilename,
         version: &Version,
         requires_python: Option<VersionSpecifiers>,
+        hashes: &[HashDigest],
         yanked: Option<Yanked>,
         excluded: bool,
         upload_time: Option<i64>,
@@ -500,8 +528,31 @@ impl VersionMapLazy {
             }
         }
 
-        // Determine a compatibility for the wheel based on tags
-        WheelCompatibility::from(filename.compatibility(&self.tags))
+        // Determine a compatibility for the wheel based on tags.
+        let priority = match filename.compatibility(&self.tags) {
+            TagCompatibility::Incompatible(tag) => {
+                return WheelCompatibility::Incompatible(IncompatibleWheel::Tag(tag))
+            }
+            TagCompatibility::Compatible(priority) => priority,
+        };
+
+        // Check if hashes line up. If hashes aren't required, they're considered matching.
+        let hash = if self.required_hashes.is_empty() {
+            Hash::Matched
+        } else {
+            if hashes.is_empty() {
+                Hash::Missing
+            } else if hashes
+                .iter()
+                .any(|hash| self.required_hashes.contains(hash))
+            {
+                Hash::Matched
+            } else {
+                Hash::Mismatched
+            }
+        };
+
+        WheelCompatibility::Compatible(hash, priority)
     }
 }
 
