@@ -14,6 +14,7 @@ use thiserror::Error;
 use distribution_types::{DistributionMetadata, IndexLocations, Name, ResolvedDist};
 use install_wheel_rs::linker::LinkMode;
 use pep508_rs::Requirement;
+use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
 use uv_client::{Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::KeyringProviderType;
@@ -119,6 +120,11 @@ async fn venv_impl(
         find_default_python(cache).into_diagnostic()?
     };
 
+    // Add all authenticated sources to the cache.
+    for url in index_locations.urls() {
+        store_credentials_from_url(url);
+    }
+
     writeln!(
         printer.stderr(),
         "Using Python {} interpreter at: {}",
@@ -134,13 +140,9 @@ async fn venv_impl(
     )
     .into_diagnostic()?;
 
-    // Extra cfg for pyvenv.cfg to specify uv version
-    let extra_cfg = vec![("uv".to_string(), env!("CARGO_PKG_VERSION").to_string())];
-
     // Create the virtual environment.
-    let venv =
-        uv_virtualenv::create_venv(path, interpreter, prompt, system_site_packages, extra_cfg)
-            .map_err(VenvError::Creation)?;
+    let venv = uv_virtualenv::create_venv(path, interpreter, prompt, system_site_packages)
+        .map_err(VenvError::Creation)?;
 
     // Install seed packages.
     if seed {
